"""
"""

import types
from Numeric import array,Float,ArrayType

DEFAULT_ZONE = 56

class Geo_reference:
    """
    assuming projection x,
    datum y
    """

    def __init__(self,
                 zone=None,
                 xllcorner=None,
                 yllcorner=None,
                 NetCDFObject = None,
                 ASCIIFile = None):
        if not xllcorner == None:
            self.xllcorner = xllcorner #Easting of lower left corner
        if not yllcorner == None:
            self.yllcorner = yllcorner #Northing of lower left corner
        if not zone == None:
            self.zone = zone
        
        if not NetCDFObject == None:
            self.read_NetCDF(NetCDFObject)

        #FIXME think if this should be here    
        if not ASCIIFile == None:
            self.read_ASCII(ASCIIFile)

    def get_xllcorner(self):
        return self.xllcorner
    
    def get_yllcorner(self):
        return self.yllcorner
    
    def get_zone(self):
        return self.zone
        
    def write_NetCDF(self, outfile):
        outfile.xllcorner = self.xllcorner 
        outfile.yllcorner = self.yllcorner
        outfile.zone = self.zone

        # might add this as well?
        #outfile.false_easting = false_easting
        #outfile.false_northing =false_northing
    
        #outfile.projection = projection
        #outfile.datum = datum
        #outfile.units = units

    def read_NetCDF(self, infile):
        self.xllcorner = infile.xllcorner[0]
        self.yllcorner = infile.yllcorner[0] 
        self.zone = infile.zone[0]

        
        assert (type(self.xllcorner) == types.FloatType or type(self.xllcorner) == types.IntType)
        assert (type(self.yllcorner) == types.FloatType or type(self.yllcorner) == types.IntType)
        assert (type(self.zone) == types.IntType)

        
    def write_ASCII(self, fd):
        fd.write('# Geo reference' + "\n")
        fd.write(str(self.zone) + "\n") 
        fd.write(str(self.xllcorner) + "\n") 
        fd.write(str(self.yllcorner) + "\n")

    def read_ASCII(self, fd):
        fd.readline() # remove the title line
        self.zone = int(fd.readline())
        self.xllcorner = float(fd.readline())
        self.yllcorner = float(fd.readline())
            
        
        assert (type(self.xllcorner) == types.FloatType)
        assert (type(self.yllcorner) == types.FloatType)
        assert (type(self.zone) == types.IntType)
        
        #false_easting = infile.false_easting[0]
        #false_northing = infile.false_northing[0]

    def change_points_geo_ref(self, points,
                       points_geo_ref=None):
        """
        Change the reference of a list or Numeric array of points.
        If the points do not have ageo ref, assume 'absolute' values
        """
        is_list = False
        if type(points) == types.ListType:
            is_list = True
            if len(points)>0 and type(points[0]) not in [types.ListType,types.TupleType]:
                #a single pont is being passed.  make it a list of lists
                points = [points]
        elif type(points) == ArrayType:
            if len(points.shape) == 1:
                points = [points]
                
            
            
        #convert into array
        points = array(points).astype(Float) 
        #add point geo ref to points
        if not points_geo_ref is None:
            points[:,0] += points_geo_ref.xllcorner 
            points[:,1] += points_geo_ref.yllcorner 
        
        #subtract primary geo ref from points
        points[:,0] -= self.xllcorner 
        points[:,1] -= self.yllcorner
        if is_list:
            points = points.tolist()
        # return points
        return points

    
    #def easting_northing2geo_reffed_point(self, x, y):
    #    return [x-self.xllcorner, y - self.xllcorner]

    #def easting_northing2geo_reffed_points(self, x, y):
    #    return [x-self.xllcorner, y - self.xllcorner]

    def get_origin(self):
        return (self.zone,self.xllcorner,self.yllcorner)
    
    def __repr__(self):
        return "(%i %f,%f)" % (self.zone,self.xllcorner,self.yllcorner)
    
    def __cmp__(self,other):
        # FIXME (DSG) add a tolerence
        if other is None:
            return 1
        cmp = 0
        if not (self.xllcorner == self.xllcorner):
            cmp = 1
        if not (self.yllcorner == self.yllcorner):
            cmp = 1
        if not (self.zone == self.zone):
            cmp = 1
        return cmp
        
#-----------------------------------------------------------------------

if __name__ == "__main__":
    pass
